@@ -1,6 +1,9 @@
+from functools import wraps
 import os
 import re
 
+from .fs import get_absolute_path
+
 # -----------------------------------------------------------------------------
 # Map function Helper functions
 # -----------------------------------------------------------------------------
@@ -135,6 +138,7 @@ def normalize_location(func):
         .Geom.Perm => Geom/Perm
 
     """
+    @wraps(func)
     def wrapper(*args, **kwargs):
         args = list(args)
         for i, arg in enumerate(args):
@@ -144,3 +148,19 @@ def normalize_location(func):
 
         return func(*args, **kwargs)
     return wrapper
+
+
+# -----------------------------------------------------------------------------
+
+def with_absolute_path(func):
+    """Assume the first string argument is a path and resolve it."""
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        args = list(args)
+        for i, arg in enumerate(args):
+            if isinstance(arg, str):
+                args[i] = get_absolute_path(arg)
+                break
+
+        return func(*args, **kwargs)
+    return wrapper
